#!/usr/bin/perl

#┌─────────────────────────────────
#│ TOPICS BOARD : topics.cgi - 2021/07/25
#│ copyright (c) kentweb, 1997-2021
#│ https://www.kent-web.com/
#└─────────────────────────────────
# REBUILD(改造)
#┌─────────────────────────────────
#│  NEWS BOOK v1.5 : index.cgi - 2023/08/06
#│  Edited by TRANSFORM Since 2012
#│  https://www.transformsite.com/
#└─────────────────────────────────
# モジュール宣言
use strict;
use CGI::Carp qw(fatalsToBrowser);
use Encode;
use lib "./lib";
use CGI::Minimal;
use Calendar::Japanese::Holiday;

# 設定ファイル認識
require "./init.cgi";
my %cf = set_init();

# データ受理
CGI::Minimal::max_read_size($cf{maxdata});
my $cgi = CGI::Minimal->new;
error('容量オーバー') if ($cgi->truncated);
my %in = parse_form($cgi);

# アクセス制限
passwd(%in) if ($cf{enter_pwd} ne '');

# 処理分岐
find_log() if ($in{find});
if ($in{mode} eq "daycalen") { daycalen_log(); }
if ($in{mode} eq "moncalen") { moncalen_log(); }
if ($in{mode} eq "topic") { topic_log(); }
if ($in{mode} eq "rescom") { rescom_form(); }
if ($in{mode} eq "res_find") { res_find_data(); }
if ($in{mode} eq "compwd") { com_pwdform(); }
if ($in{mode} eq "com_list") { com_list(); }
if ($in{mode} eq 'source_article') { article_data(); }

bbs_list();

#-----------------------------------------------------------
#  記事表示
#-----------------------------------------------------------
sub bbs_list {
	# ページ数
	my $pg = $in{pg} or 0;

	# プロフィール認識
	my %set = read_profile();

	# コメント数カウント
	my ($i,@log,%res_cnt,@reply,@main_log,@rescnt);
	open(IN,"$cf{rescnt}");
	while(<IN>) {
		my ($no,$y,$m,$d,$res) = split(/<>/);

		$res_cnt{$no} = $res;

	}

	# 書庫ファイル
	open(IN,"$cf{datadir}/archive.dat") or error("open err: archive.dat");
	my @archi = <IN>;
	close(IN);
	
	chomp @archi;
	
	# 変数初期化
	my ($caly,$calm,%cal,$pager,@log);

	# 初期表示
	if (!$in{mode}) {
		
		my ($y,$m,$cnt) = split(/,/,$archi[0]);
		$caly = $y;
		$calm = $m;

		# データ認識
		my ($i,%log,%num);
		open(IN,"$cf{datadir}/index.dat") or &error("open err: index.dat");
		while(<IN>) {
			my ($no,$y,$m,$d,$tm) = (split(/<>/))[0,1,2,3,4];
			$i++;

			next if ($i < $pg + 1);
			next if ($i > $pg + $cf{pg_max});

			$log{"$y$m"}++;
			$num{$no}++;
		}
		close(IN);
	
		$pager = make_pager($i,$pg);

		# 月次データ読込
		for ( sort{ $b <=> $a } keys %log ) {
			/^(\d{4})(\d{2})/;
			open(IN,"$cf{datadir}/log/$1-$2.dat");
			while(<IN>) {
				my ($no,$y,$m,$d,$time,$sub,$com,$ex1,$w1,$h1,$ex2,$w2,$h2,$ex3,$w3,$h3,$tag,$att,$tube,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$cm) = split(/<>/);

				# トップ表示記事
				if (defined $num{$no}) { push(@log,$_); }
			}
			close(IN);
		}

	# アーカイブ記事
	} elsif ($in{mode} eq "archive") {
		
		# 年月認識
		($caly,$calm) = $in{ym} =~ /^(\d{4})(\d{2})$/ ? ($1,$2)	: error("年月不正");
		
		# 当月記事
		my $i = 0;
		open(IN,"$cf{datadir}/log/$caly-$calm.dat") ;
		while(<IN>) {
			$i++;
			my ($no,$y,$m,$d,$time,$sub,$com,$ex1,$w1,$h1,$ex2,$w2,$h2,$ex3,$w3,$h3,$tag,$att,$tube,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$cm) = split(/<>/);
			
			# カレンダ用
			$cal{$d}++;
			
			# 日指定の場合
			next if ($in{d} > 0 && $in{d} != $d);
			
				next if ($i < $pg + 1);
				next if ($i > $pg + $cf{pg_max});
				
				push(@log,$_);
		}
		close(IN);
		
		# ペイジャー作成
		$pager = make_pager($i,$pg);
	}

	# Recent Posts data
	my $topic = read_topic();

	# Recent comments data
	my $comlist = read_com();

	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/bbs.html") or error("open err: bbs.html");
	my $tmpl = join('',<IN>);
	close(IN);
	
	$tmpl =~ s/!(bbs_title|homepage|cmnurl|[a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/<!-- pager -->/$pager/g;

	my $prof_img = $set{prof_image}
			? qq|<p><img src="$cf{uplurl}/$set{prof_image}" style="width:$set{prof_img_w}px" alt="プロフ"></p>| : '';

	$tmpl =~ s|<!-- calen -->(.+?)<!-- /calen -->|put_calen($1,$caly,$calm,\%cal,\@archi)|se;
	$tmpl =~ s|<!-- archive -->(.+?)<!-- /archive -->|put_archive($1,\@archi)|se;
	$tmpl =~ s/!topic_count!/$cf{max_topics}/g;
	$tmpl =~ s/<!-- topic -->/$topic/;

	$set{profile} =~ s/\t/<br>/;
	$tmpl =~ s/!profile!/$set{profile}/g;
	$tmpl =~ s|<!-- prof_image -->|$prof_img|g;

	open(DAT,"$cf{datadir}/resno.dat");
	my $number = join('',<DAT>);
	close(DAT);

	if ($number > 0) {
		$tmpl =~ s/<!-- comlist -->/$comlist/;
		$tmpl =~ s/!comment_count!/$cf{max_com}/g;
	} else {
		$tmpl =~ s|<!-- com_link -->(.+?)<!-- /com_link -->||s;
	}

	# アーカイブ記事
	my ($yr,$mon) = $in{ym} =~ /^(\d{4})(\d{2})/ && ($1,$2);
	$in{ym} = "$yr年$mon月";

	if ($in{mode} eq 'archive' && $in{ym}) {

		$tmpl =~ s|<!-- head-ttl -->|<div class="art-title">アーカイブ記事 ( $in{ym} )</div>|g;
		$tmpl =~ s|<!-- head-btn -->|<div class="ta-r"><button type="button" onclick="javascript:window.location='!bbs_cgi!'" class="large button">Back to the TOP</button></div>|g;
		$tmpl =~ s/!bbs_cgi!/$cf{bbs_cgi}/g;
	}

	# プロフィール
	if ($cf{prof} == 0) {
		$tmpl =~ s|<!-- profile -->(.+?)<!-- /profile -->||s;
	}
	
	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ m|(.+)<!-- loop -->(.+?)<!-- /loop -->(.+)|s
			? ($1,$2,$3)
			: error("テンプレート不正");
	
	# ヘッダ表示
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;
	
	# 記事展開
	for (@log) {
		my ($no,$y,$m,$d,$time,$sub,$com,$ex1,$w1,$h1,$ex2,$w2,$h2,$ex3,$w3,$h3,$tag,$att,$tube,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$cm) = split(/<>/);

		if (!$res_cnt{$no}) { $res_cnt{$no} = 0; }

		$com =~ s/\t/<br>/g;
		if ($tag == 1) {
			$com = tag($com);
		} elsif ($cf{autolink}) {
			$com = auto_link($com);
		}
		
		# YouTube
		my $clip;
		if ($att eq 't') {
			$clip = qq|<div class="youtube">| . tag($tube) . qq|</div>| if ($tube);
		
		# UPファイル
		} else {
			$clip = attach($no,$ex1,$w1,$h1,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5);
		}
		$clip &&= "<p>$clip</p>";
		
		# 文字置換
		my $tmp = $loop;
		my $day = "$y/$m/$d $time";
		$tmp =~ s/!date!/$day/g;
		$tmp =~ s/!id!/$no/g;
		$tmp =~ s/!subject!/$sub/g;
		$tmp =~ s/!comment!/$com/g;
		$tmp =~ s/<!-- clip -->/$clip/g;
		$tmp =~ s/!res_com!/$cf{bbs_cgi}?mode=rescom&no=$no/g;
		$tmp =~ s/!ico_com!/$cf{ico_com}/g;
		$tmp =~ s/!res_cnt!/$res_cnt{$no}/g;

		# コメント返信機能ＯＦＦ
		$tmp =~ s/<!-- cmlink -->.+?<!-- cmlink -->//s if (!$cm);

		print $tmp;
	}
	
	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  最新記事INDEX作成
#-----------------------------------------------------------
sub read_topic {

	# 最新記事リスト表示
	my ($i,$topic,@top_ttl,%ym);
	open(IN,"$cf{datadir}/index.dat") or error("open err: index.dat");
	while(<IN>) {
		my ($no,$y,$m,$d,$time,$sub,$com,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$tag,$att,$tube,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$cm) = split(/<>/);

			if (defined($no)) { push(@top_ttl,$_); }

		# 文字カット
		if (length($sub) > 25) {
			$sub = substr($sub,0,27) . '...';
		}
		
		$i++;
		last if ($i >= $cf{max_topics} + 1);

		# 年月
		$ym{"$y$m"}++;

		$topic .= qq|<div id="topi-ttl"><span style="color:#2ca9e1">●</span> <a href="$cf{bbs_cgi}?mode=topic&no=$no&ym=$y$m"><span style="font-size:9pt">$sub</a> ($m/$d $time)</span></div>\n|;
	}
	close(IN);

	foreach my $ttl (@top_ttl) {
		my ($no,$y,$m,$d,$time,$sub,$com,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$tag,$att,$tube,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$cm) = split(/<>/, $ttl);

			$topic = qq|$topic\n|;
	}
	# 結果を返す
	return $topic;
}

#-----------------------------------------------------------
#  最新記事リスト・記事閲覧
#-----------------------------------------------------------
sub topic_log {

	# プロフィール認識
	my %set = read_profile();

	# コメント数カウント
	my %res_cnt;
	open(IN,"$cf{rescnt}");
	while(<IN>) {
		my ($no,$y,$m,$d,$res) = split(/<>/);

		$res_cnt{$no} = $res;

	}

	my (@log);
	open(IN,"$cf{datadir}/index.dat") or &error("open err: index.dat");
	while (<IN>) {
		chomp;
		my ($no,$y,$m,$d,$time,$sub,$com,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$tag,$att,$tube,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$cm) = split(/<>/);
		if ($in{no} == $no) {
			push(@log,$_);
		}
	}
	close(IN);

	# Recent Posts data
	my $topic = read_topic();

	# Recent comments data
	my $comlist = read_com();

	# テンプレート認識
	open(IN,"$cf{tmpldir}/topic.html") or &error("open err: topic.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# 書庫ファイル
	open(IN,"$cf{datadir}/archive.dat") or error("open err: archive.dat");
	my @archi = <IN>;
	close(IN);
	
	chomp @archi;

	# 変数初期化
	my ($caly,$calm,%cal);

	# 日付フォーマット
	my ($y,$m) = $in{ym} =~ /^(\d{4})(\d{2})/ && ($1,$2);
		$caly = $y;
		$calm = $m;

	$tmpl =~ s|<!-- calen -->(.+?)<!-- /calen -->|put_calen($1,$caly,$calm,\%cal,\@archi)|se;
	$tmpl =~ s|<!-- archive -->(.+?)<!-- /archive -->|put_archive($1,\@archi)|se;
	$tmpl =~ s/!topic_count!/$cf{max_topics}/g;
	$tmpl =~ s/<!-- topic -->/$topic/;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
	$tmpl =~ s|!cmnurl!|$cf{cmnurl}|g;
	$tmpl =~ s/!homepage!/$cf{homepage}/g;

	my $prof_img = $set{prof_image}
			? qq|<p><img src="$cf{uplurl}/$set{prof_image}" style="width:$set{prof_img_w}px" alt="プロフ"></p>| : '';

	$set{profile} =~ s/\t/<br>/;
	$tmpl =~ s/!profile!/$set{profile}/g;
	$tmpl =~ s|<!-- prof_image -->|$prof_img|g;

	open(DAT,"$cf{datadir}/resno.dat");
	my $number = join('',<DAT>);
	close(DAT);

	if ($number > 0) {
		$tmpl =~ s/<!-- comlist -->/$comlist/;
		$tmpl =~ s/!comment_count!/$cf{max_com}/g;
	} else {
		$tmpl =~ s/<!-- cmlink -->.+?<!-- cmlink -->//s;
	}

	# プロフィール
	if ($cf{prof} == 0) {
		$tmpl =~ s|<!-- profile -->(.+?)<!-- /profile -->||s;
	}

	# 日付フォーマット
	my $day = "$y年$m月";
	$tmpl =~ s/!day!/$day/g;

	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ m|(.+)<!-- loop -->(.+?)<!-- /loop -->(.+)|s
			? ($1,$2,$3)
			: error("テンプレート不正");

	# 文字置換
	for ($head, $foot) {
		s/!bbs_title!/$cf{bbs_title}/g;
		s/!subttl!/$cf{sub_ttl}/g;
		s/!([a-z]+_cgi)!/$cf{$1}/g;
		s/!homepage!/$cf{homepage}/g;
		s|!rss!|$cf{htmlurl}/index.xml|;
	}

	# ヘッダ表示
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;
	
	# 記事展開
	for (@log) {
		my ($no,$y,$m,$d,$time,$sub,$com,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$tag,$att,$tube,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$cm) = split(/<>/);
		$com =~ s/\t/<br>/g;
		if ($tag == 1) {
			$com = tag($com);
		} elsif ($cf{autolink}) {
			$com = auto_link($com);
		}
		
		# YouTube
		my $clip;
		if ($att eq 't') {
			$clip = qq|<div class="youtube">| . tag($tube) . qq|</div>| if ($tube);
		
		# UPファイル
		} else {
			$clip = attach($no,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5);
		}
		$clip &&= "<p>$clip</p>";
		
		# 文字置換
		my $tmp = $loop;
		my $day = "$y/$m/$d $time";
		$tmp =~ s/!date!/$day/g;
		$tmp =~ s/!subject!/$sub/g;
		$tmp =~ s/!comment!/$com/g;
		$tmp =~ s/<!-- clip -->/$clip/g;
		$tmp =~ s/!id!/$no/g;
		$tmp =~ s/!res_cnt!/$res_cnt{$no}/g;

		# コメント返信機能ＯＦＦ
		$tmp =~ s/<!-- cmlink -->.+?<!-- cmlink -->//s if (!$cm);
		print $tmp;
	}

	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  コメントINDEX作成
#-----------------------------------------------------------
sub read_com {

	# 最新コメントリスト表示
	my ($i,$comlist,@top_com,@data);
	open(IN,"$cf{reslog}") or error("open err: $cf{reslog}");
	my @data = <IN>;
	close(IN);

	#@data = reverse(@data);

	foreach(@data) {
		my ($no,$ent,$sub,$date,$name,$url,$host,$com,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$chk,$pw,$comsub) = split(/<>/);

		if (defined($no)) { push(@top_com,$_); }

		# 文字カット
		if (length($comsub) > 23) {
			$comsub = substr($comsub,0,30) . '...略';
		}

		$i++;
		last if ($i >= $cf{max_com} + 1);

		my ($y,$m,$d) = $date =~ /^(\d{4})\/(\d{2})\/(\d{2})/ && ($1,$2,$3);
		my $day = "$m/$d ";
		
		$comlist .= qq|<div id="topi-ttl"><span style="color:#2ca9e1">●</span> <span style="color:#7d993d;">$name</span> -&nbsp;&nbsp;<a href="$cf{bbs_cgi}?mode=com_list&no=$no">$comsub</a> ($day)</div>\n|;
	}

	foreach my $ttl (@top_com) {
		my ($no,$ent,$sub,$date,$name,$url,$host,$com,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$chk,$pw,$comsub) = split(/<>/, $ttl);

			$comlist = qq|$comlist\n|;
	}
	# 結果を返す
	return $comlist;
}

#-----------------------------------------------------------
#  最新コメントリスト・記事閲覧
#-----------------------------------------------------------
sub com_list {

	# コメント数カウント
	my %res_cnt;
	open(IN,"$cf{rescnt}");
	while(<IN>) {
		my ($no,$y,$m,$d,$res) = split(/<>/);

		$res_cnt{$no} = $res;

	}

	my (@reply);
	open(IN,"$cf{reslog}") or error("open err: $cf{reslog}");
	while (<IN>) {
		chomp;
		my ($no,$ent,$sub,$date,$name,$url,$host,$com,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$chk,$pw,$comsub) = split(/<>/);
		if ($in{no} == $no) {
			push(@reply,$_);
		}
	}
	close(IN);

	# Recent Posts data
	my $topic = read_topic();

	# Recent comments data
	my $comlist = read_com();

	# テンプレート認識
	open(IN,"$cf{tmpldir}/com_list.html") or &error("open err: com_list.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# 書庫ファイル
	open(IN,"$cf{datadir}/archive.dat") or error("open err: archive.dat");
	my @archi = <IN>;
	close(IN);
	
	chomp @archi;

	# 変数初期化
	my ($caly,$calm,%cal);

	$tmpl =~ s|<!-- archive -->(.+?)<!-- /archive -->|put_archive($1,\@archi)|se;
	$tmpl =~ s/!topic_count!/$cf{max_topics}/g;
	$tmpl =~ s/<!-- topic -->/$topic/;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/style.css|g;
	$tmpl =~ s|!cmnurl!|$cf{cmnurl}|g;
	$tmpl =~ s/!homepage!/$cf{homepage}/g;
	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s|!icon:(\w+\.\w+)!|<img src="$cf{iconurl}/$1" alt="" class="icon" />|g;

	open(DAT,"$cf{datadir}/resno.dat");
	my $number = join('',<DAT>);
	close(DAT);

	if ($number > 0) {
		$tmpl =~ s/<!-- comlist -->/$comlist/;
		$tmpl =~ s/!comment_count!/$cf{max_com}/g;
	} else {
		$tmpl =~ s/<!-- cmlink -->.+?<!-- cmlink -->//s;
	}

	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ m|(.+)<!-- loop -->(.+?)<!-- /loop -->(.+)|s
			? ($1,$2,$3)
			: error("テンプレート不正");

	# 文字置換
	for ($head, $foot) {
		s/!bbs_title!/$cf{bbs_title}/g;
		s/!([a-z]+_cgi)!/$cf{$1}/g;
		s/!homepage!/$cf{homepage}/g;
		s|!rss!|$cf{htmlurl}/index.xml|;
	}

	# ヘッダ表示
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;
	
	# 記事展開
	foreach (@reply) {
		my ($no,$ent,$sub,$date,$name,$url,$host,$com,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$chk,$pw,$comsub) = split(/<>/);

		if ($url) { $url = qq |&lt;<a href="$url" target="_blank">URL</a>&gt;|; }

			$com = auto_link($com);
		
		# UPファイル
		my $resclip;
			$resclip = res_attach($no,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3);
			$resclip &&= "<p>$resclip</p>";

		# 文字置換
		my $tmp = $loop;
		$tmp =~ s/!r_num!/$no/g;
		$tmp =~ s/!r_name!/$name/g;
		$tmp =~ s/!r_sub!/$comsub/g;
		$tmp =~ s/!r_date!/$date/g;
		$tmp =~ s/!r_url!/$url/g;
		$tmp =~ s/!r_comment!/$com/g;
		$tmp =~ s/<!-- ext -->/$resclip/g;
		$tmp =~ s/!entry!/$ent/g;
		$tmp =~ s/!sub!/$sub/g;
		$tmp =~ s/!res_cnt!/$res_cnt{$no}/g;
		$tmp =~ s|!com_edit!|<a href="$cf{bbs_cgi}?mode=compwd&no=$no"><img src="$cf{iconurl}/edit03.png" alt="" class="icon" /></a>|g;

		print $tmp;
	}

	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  コメント元記事閲覧 (コメントリスト個別表示対応)
#-----------------------------------------------------------
sub article_data {

	my (@log,%yr,%mon);
	open(IN,"$cf{datadir}/index.dat") or &error("open err: index.dat");
	while (<IN>) {
		chomp;
		my ($no,$y,$m,$d,$time,$sub,$com,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$tag,$att,$tube,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$cm) = split(/<>/);

		if ($in{no} == $no) {
			push(@log,$_);
		}

		$yr{$no} = $y;
		$mon{$no} = $m;

	}
	close(IN);

	# レス{コメント}カウント
	my ($i,@data,%res_cnt,%cat);
	open(IN,"$cf{rescnt}");
	while(<IN>) {
		my ($no,$y,$m,$d,$res) = split(/<>/);

		$res_cnt{$no} = $res;

	}

	# 書庫ファイル
	open(IN,"$cf{datadir}/archive.dat") or error("open err: archive.dat");
	my @archi = <IN>;
	close(IN);
	
	chomp @archi;

	# 変数初期化
	my ($caly,$calm,%cal);
		$caly = $yr{$in{no}};
		$calm = $mon{$in{no}};

	# Recent Posts data
	my $topic = read_topic();

	# Recent comments data
	my $comlist = read_com();

	# テンプレート認識
	open(IN,"$cf{tmpldir}/article.html") or &error("open err: article.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# 文字置き換え
	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s|!icon:(\w+\.\w+)!|<img src="$cf{iconurl}/$1" alt="" class="icon" />|g;
	$tmpl =~ s|<!-- calen -->(.+?)<!-- /calen -->|put_calen($1,$caly,$calm,\%cal,\@archi)|se;
	$tmpl =~ s|<!-- archive -->(.+?)<!-- /archive -->|put_archive($1,\@archi)|se;
	$tmpl =~ s/<!-- topic -->/$topic/;
	$tmpl =~ s/<!-- comment -->/$comlist/;
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/topics.css|g;
	$tmpl =~ s|!cmnurl!|$cf{cmnurl}|g;
	$tmpl =~ s|!bbs_cgi!|$cf{bbs_cgi}|g;
	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/!homepage!/$cf{homepage}/g;

	open(DAT,"$cf{datadir}/resno.dat");
	my $number = join('',<DAT>);
	close(DAT);

	if ($number > 0) {
		$tmpl =~ s/<!-- comlist -->/$comlist/;
		$tmpl =~ s/!comment_count!/$cf{max_com}/g;
	} else {
		$tmpl =~ s/<!-- cmlink -->.+?<!-- cmlink -->//s;
	}

	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ m|(.+)<!-- loop -->(.+?)<!-- /loop -->(.+)|s
			? ($1,$2,$3)
			: error("テンプレート不正");

	# 文字置換
	for ($head, $foot) {
		s/!bbs_title!/$cf{bbs_title}/g;
		s/!subttl!/$cf{sub_ttl}/g;
		s/!([a-z]+_cgi)!/$cf{$1}/g;
		s/!homepage!/$cf{homepage}/g;
		s|!rss!|$cf{htmlurl}/index.xml|;
	}

	# ヘッダ表示
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;
	
	# 記事展開
	foreach (@log) {
		my ($no,$y,$m,$d,$time,$sub,$com,$ex1,$w1,$h1,$ex2,$w2,$h2,$ex3,$w3,$h3,$tag,$att,$tube,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$cm) = split(/<>/);

		if (!$res_cnt{$no}) { $res_cnt{$no} = 0; }

		$com =~ s/\t/<br>/g;
		if ($tag == 1) {
			$com = tag($com);
		} elsif ($cf{autolink}) {
			$com = auto_link($com);
		}
		
		# YouTube
		my $clip;
		if ($att eq 't') {
			$clip = qq|<div class="youtube">| . tag($tube) . qq|</div>| if ($tube);
		
		# UPファイル
		} else {
			$clip = attach($no,$ex1,$w1,$h1,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5);
		}
		$clip &&= "<p>$clip</p>";
		
		# 文字置換
		my $tmp = $loop;
		my $day = "$y/$m/$d $time";
		$tmp =~ s/!date!/$day/g;
		$tmp =~ s/!id!/$no/g;
		$tmp =~ s/!subject!/$sub/g;
		$tmp =~ s/!comment!/$com/g;
		$tmp =~ s/<!-- clip -->/$clip/g;
		$tmp =~ s/!res_com!/$cf{bbs_cgi}?mode=rescom&no=$no/g;
		$tmp =~ s/!ico_com!/$cf{ico_com}/g;
		$tmp =~ s/!res_cnt!/$res_cnt{$no}/g;

		# コメント返信機能ＯＦＦ
		$tmp =~ s/<!-- cmlink -->.+?<!-- cmlink -->//s if (!$cm);
		print $tmp;
	}
	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  メニューアーカイブ
#-----------------------------------------------------------
sub put_archive {
	my ($loop,$archi) = @_;
	
	my $ret;
	for (@$archi) {
		my ($y,$m,$cnt) = split(/,/);
		
		my $tmp = $loop;
		$tmp =~ s/!year!/$y/g;
		$tmp =~ s/!month!/$m/g;
		$tmp =~ s/!count!/$cnt/g;
		$tmp =~ s/!uri!/$cf{bbs_cgi}?mode=archive&ym=$y$m/g;
		
		$ret .= $tmp;
	}
	return $ret;
}

#-----------------------------------------------------------
#  メニューカレンダー
#-----------------------------------------------------------
sub put_calen {
	my ($calen,$y,$m,$cal,$arcv) = @_;
	
	$in{y} = $y;
	$in{m} = $m;

	# アーカイブ
	my ($pt,$next,$prev);
	my $i = 0;
	for (@$arcv) {
		my ($yr,$mon,$cnt) = split(/,/);
		
		if ($y == $yr && $m == $mon) {
			$pt = $i;
		}
		$i++;
	}
	# 当月のみ
	if ($i == 0 or $i == 1) {
		$prev = '&nbsp;';
		$next = '&nbsp;';
	
	# TOP
	} elsif ($pt == 0) {
		my ($yr,$mon,$cnt) = split(/,/,$$arcv[1]);

		$prev = qq|<a href="$cf{bbs_cgi}?mode=moncalen&ym=$yr$mon">&laquo;</a>|;
		$next = '&nbsp;';
	
	# 最終
	} elsif ($pt == $#{$arcv}) {
		my ($yr,$mon,$cnt) = split(/,/,$$arcv[$#{$arcv} - 1]);
		
		$prev = '&nbsp;';
		$next = qq|<a href="$cf{bbs_cgi}?mode=moncalen&ym=$yr$mon">&raquo;</a>|;
	
	# 中間
	} else {
		my ($yr,$mon,$cnt) = split(/,/,$$arcv[$pt + 1]);
		$prev = qq|<a href="$cf{bbs_cgi}?mode=moncalen&ym=$yr$mon">&laquo;</a>|;
		
		my ($yr,$mon,$cnt) = split(/,/,$$arcv[$pt - 1]);
		$next = qq|<a href="$cf{bbs_cgi}?mode=moncalen&ym=$yr$mon">&raquo;</a>|;
	}
	
	if (%$cal == 0) {
		open(IN,"$cf{datadir}/log/$y-$m.dat");
		while(<IN>) {
			my ($no,$y,$m,$d,$time,$sub,$com,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$tag,$att,$tube,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5) = split(/<>/);
			
			$$cal{$d}++;
		}
		close(IN);
	}
	
	my ($top,$cel,$bot) = $calen =~ m|(.+)<!-- days -->(.+?)<!-- /days -->(.+)|s
			? ($1,$2,$3)
			: return 'bad calendar';
	
	for ($top,$bot) {
		s/!year!/$y/g;
		s/!month!/$m/g;
		s/<!-- prev -->/$prev/g;
		s/<!-- next -->/$next/g;
	}
	
	# 初日の曜日を取得
	my $top_w = get_week($in{y},$in{m},1);
	
	# 当月の末日を求める
	my $last = last_day($in{y},$in{m});

	# 祝日を認識
	my $hdays = getHolidays($in{y},$in{m},1);
	my %hol;
	while ( my ($key,$val) = each %{$hdays} ) {
		$hol{$key} = encode('utf8',$val);
	}
	
	# 第1週：月初のブランクを記述
	my $tmp = $cel;
	my ($ret,$flg);
	my $day = 0;
	for my $w (0 .. 6) {
		if ($w == $top_w) {
			$flg++;
		}
		if ($flg) {
			$day++;

				# 祝日のときは背景色を変える
				if (defined $hol{$day}) {
					$tmp =~ s/<td>!$w!/<td style="background:$cf{week_col}" title="$hol{$day}">!$w!/;
				}

			my $d = sprintf("%02d",$day);
			# 記事あり
			if (defined $$cal{$d}) {
				$tmp =~ s|!$w!|<a href="$cf{bbs_cgi}?mode=daycalen&amp;ym=$y$m&amp;d=$d">$day</a>|;
			# 記事なし
			} else {
				$tmp =~ s/!$w!/$day/;
			}

		} else {
			$tmp =~ s/!$w!/&nbsp;/;
		}
	}
	$ret .= $tmp;
	
	# 第2週以降
	my $flg;
	while (!$flg) {
		my $tmp = $cel;
		for my $w (0 .. 6) {
			if (!$flg) {
				$day++;

				# 祝日のときは背景色を変える
				if (defined $hol{$day}) {
					$tmp =~ s/<td>!$w!/<td style="background:$cf{week_col}" title="$hol{$day}">!$w!/;
				}

				my $d = sprintf("%02d",$day);

				# 記事あり
				if (defined $$cal{$d}) {
					$tmp =~ s|!$w!|<a href="$cf{bbs_cgi}?mode=daycalen&amp;ym=$y$m&amp;d=$d">$day</a>|;
				# 記事なし
				} else {
					$tmp =~ s/!$w!/$day/;
				}
			} else {
				$tmp =~ s/!$w!/&nbsp;/;
			}
			if ($day >= $last) {
				$flg = 1;
			}
		}
		$ret .= $tmp;
	}
	return $top . $ret . $bot;
}

#-----------------------------------------------------------
#  月の末日
#-----------------------------------------------------------
sub last_day {
	my ($yy,$mm) = @_;
	
	return (31,28,31,30,31,30,31,31,30,31,30,31) [$mm - 1]
		+ ($mm == 2 && (($yy % 4 == 0 && $yy % 100 != 0) || $yy % 400 == 0));
}

#-----------------------------------------------------------
#  週算出
#-----------------------------------------------------------
sub get_week {
	my ($yy,$mm,$dd) = @_;
	
	if ($mm == 1 || $mm == 2) {
		$yy--;
		$mm += 12;
	}
	int( $yy + int($yy / 4) - int($yy / 100) + int($yy / 400) + int((13 * $mm + 8) / 5) + $dd ) % 7;
}

#-----------------------------------------------------------
#  カレンダー日次記事
#-----------------------------------------------------------
sub daycalen_log {

	# ページ数
	my $pg = $in{pg} or 0;

	# コメント数カウント
	my ($i,@log,%res_cnt,@reply,@main_log,@rescnt);
	open(IN,"$cf{rescnt}");
	while(<IN>) {
		my ($no,$y,$m,$d,$res) = split(/<>/);

		$res_cnt{$no} = $res;

	}

	# プロフィール認識
	my %set = read_profile();

	# 書庫ファイル
	open(IN,"$cf{datadir}/archive.dat") or error("open err: archive.dat");
	my @archi = <IN>;
	close(IN);
	
	chomp @archi;
	
	# 変数初期化
	my ($caly,$calm,%cal,$pager,@log);

	if (!$in{mode}) {
		
		my ($y,$m,$cnt) = split(/,/,$archi[0]);
		$caly = $y;
		$calm = $m;

		# データ認識
		my ($i,%log,%num);
		open(IN,"$cf{datadir}/index.dat") or &error("open err: index.dat");
		while(<IN>) {
			my ($no,$y,$m,$d,$tm) = (split(/<>/))[0,1,2,3,4];
			$i++;

			next if ($i < $pg + 1);
			next if ($i > $pg + $cf{pg_max});

			$log{"$y$m"}++;
			$num{$no}++;
		}
		close(IN);
	
		$pager = make_pager($i,$pg);

		# 月次データ読込
		for ( sort{ $b <=> $a } keys %log ) {
			/^(\d{4})(\d{2})/;
			open(IN,"$cf{datadir}/log/$1-$2.dat");
			while(<IN>) {
				my ($no,$y,$m,$d,$time,$sub,$com,$ex1,$w1,$h1,$ex2,$w2,$h2,$ex3,$w3,$h3,$tag,$att,$tube,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$cm) = split(/<>/);
				
				# トップ表示記事
				if (defined $num{$no}) { push(@log,$_); }
			}
			close(IN);
		}

	# カレンダー日次記事
	} elsif ($in{mode} eq "daycalen") {
		
		# 年月認識
		($caly,$calm) = $in{ym} =~ /^(\d{4})(\d{2})$/ ? ($1,$2)	: error("年月不正");
		
		# 当月記事
		my $i = 0;
		open(IN,"$cf{datadir}/log/$caly-$calm.dat") ;
		while(<IN>) {
			$i++;
			my ($no,$y,$m,$d,$time,$sub,$com,$ex1,$w1,$h1,$ex2,$w2,$h2,$ex3,$w3,$h3,$tag,$att,$tube,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$cm) = split(/<>/);
			
			# カレンダ用
			$cal{$d}++;
			
			# 日指定の場合
			next if ($in{d} > 0 && $in{d} != $d);
			
				next if ($i < $pg + 1);
				next if ($i > $pg + $cf{pg_max});
				
				push(@log,$_);
		}
		close(IN);
		
		# ペイジャー作成
		$pager = make_pager($i,$pg);

	}

	# Recent Posts data
	my $topic = read_topic();

	# Recent comments data
	my $comlist = read_com();

	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/day_calen.html") or error("open err: day_calen.html");
	my $tmpl = join('',<IN>);
	close(IN);
	
	$tmpl =~ s/!(bbs_title|homepage|cmnurl|[a-z]+_cgi)!/$cf{$1}/g;

	if ($cf{rss_make} == 1) {
		$tmpl =~ s|<!-- rss -->|<a href="$cf{rssurl}"><img src="$cf{cmnurl}/rss.png" class="icon"></a>|;
	}

	$tmpl =~ s|<!-- calen -->(.+?)<!-- /calen -->|put_calen($1,$caly,$calm,\%cal,\@archi)|se;
	$tmpl =~ s|<!-- archive -->(.+?)<!-- /archive -->|put_archive($1,\@archi)|se;
	$tmpl =~ s/!topic_count!/$cf{max_topics}/g;
	$tmpl =~ s/<!-- topic -->/$topic/;
	$tmpl =~ s/<!-- pager -->/$pager/g;

	my $prof_img = $set{prof_image}
			? qq|<p><img src="$cf{uplurl}/$set{prof_image}" style="width:$set{prof_img_w}px" alt="プロフ"></p>| : '';

	$set{profile} =~ s/\t/<br>/;
	$tmpl =~ s/!profile!/$set{profile}/g;
	$tmpl =~ s|<!-- prof_image -->|$prof_img|g;

	open(DAT,"$cf{datadir}/resno.dat");
	my $number = join('',<DAT>);
	close(DAT);

	if ($number > 0) {
		$tmpl =~ s/<!-- comlist -->/$comlist/;
		$tmpl =~ s/!comment_count!/$cf{max_com}/g;
	} else {
		$tmpl =~ s/<!-- cmlink -->.+?<!-- cmlink -->//s;
	}

	# プロフィール
	if ($cf{prof} == 0) {
		$tmpl =~ s|<!-- profile -->(.+?)<!-- /profile -->||s;
	}

	# 日付フォーマット
	my $d = $in{d} =~ /^(\d+)/ && ($1);
	my $week = get_week($caly,$calm,$d);
	my $day = "$caly年$calm月$d日（$cf{week}[$week]）";

	$tmpl =~ s/!day!/$day/g;
	
	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ m|(.+)<!-- loop -->(.+?)<!-- /loop -->(.+)|s
			? ($1,$2,$3)
			: error("テンプレート不正");
	
	# ヘッダ表示
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;
	
	# 記事展開
	for (@log) {
		my ($no,$y,$m,$d,$time,$sub,$com,$ex1,$w1,$h1,$ex2,$w2,$h2,$ex3,$w3,$h3,$tag,$att,$tube,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$cm) = split(/<>/);

		if (!$res_cnt{$no}) { $res_cnt{$no} = 0; }

		$com =~ s/\t/<br>/g;
		if ($tag == 1) {
			$com = tag($com);
		} elsif ($cf{autolink}) {
			$com = auto_link($com);
		}
		
		# YouTube
		my $clip;
		if ($att eq 't') {
			$clip = qq|<div class="youtube">| . tag($tube) . qq|</div>| if ($tube);
		
		# UPファイル
		} else {
			$clip = attach($no,$ex1,$w1,$h1,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5);
		}
		$clip &&= "<p>$clip</p>";
		
		# 文字置換
		my $tmp = $loop;
		my $day = "$y/$m/$d $time";
		$tmp =~ s/!date!/$day/g;
		$tmp =~ s/!subject!/$sub/g;
		$tmp =~ s/!comment!/$com/g;
		$tmp =~ s/<!-- clip -->/$clip/g;
		$tmp =~ s/!id!/$no/g;
		$tmp =~ s/!res_com!/$cf{bbs_cgi}?mode=rescom&no=$no/g;
		$tmp =~ s/!res_cnt!/$res_cnt{$no}/g;

		# コメント返信機能ＯＦＦ
		$tmp =~ s/<!-- cmlink -->.+?<!-- cmlink -->//s if (!$cm);

		print $tmp;
	}
	
	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  カレンダー月次記事
#-----------------------------------------------------------
sub moncalen_log {

	# ページ数
	my $pg = $in{pg} or 0;

	# コメント数カウント
	my ($i,@log,%res_cnt,@reply,@main_log,@rescnt);
	open(IN,"$cf{rescnt}");
	while(<IN>) {
		my ($no,$y,$m,$d,$res) = split(/<>/);

		$res_cnt{$no} = $res;

	}

	# プロフィール認識
	my %set = read_profile();

	# 書庫ファイル
	open(IN,"$cf{datadir}/archive.dat") or error("open err: archive.dat");
	my @archi = <IN>;
	close(IN);
	
	chomp @archi;
	
	# 変数初期化
	my ($caly,$calm,%cal,$pager,@log);

	if (!$in{mode}) {
		
		my ($y,$m,$cnt) = split(/,/,$archi[0]);
		$caly = $y;
		$calm = $m;

		# データ認識
		my ($i,%log,%num);
		open(IN,"$cf{datadir}/index.dat") or &error("open err: index.dat");
		while(<IN>) {
			my ($no,$y,$m,$d,$tm) = (split(/<>/))[0,1,2,3,4];
			$i++;

			next if ($i < $pg + 1);
			next if ($i > $pg + $cf{pg_max});

			$log{"$y$m"}++;
			$num{$no}++;
		}
		close(IN);
	
		$pager = make_pager($i,$pg);

		# 月次データ読込
		for ( sort{ $b <=> $a } keys %log ) {
			/^(\d{4})(\d{2})/;
			open(IN,"$cf{datadir}/log/$1-$2.dat");
			while(<IN>) {
				my ($no,$y,$m,$d,$time,$sub,$com,$ex1,$w1,$h1,$ex2,$w2,$h2,$ex3,$w3,$h3,$tag,$att,$tube,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$cm) = split(/<>/);
				
				# トップ表示記事
				if (defined $num{$no}) { push(@log,$_); }
			}
			close(IN);
		}

	# カレンダー月次記事
	} elsif ($in{mode} eq "moncalen") {
		
		# 年月認識
		($caly,$calm) = $in{ym} =~ /^(\d{4})(\d{2})$/ ? ($1,$2)	: error("年月不正");
		
		# 当月記事
		my $i = 0;
		open(IN,"$cf{datadir}/log/$caly-$calm.dat") ;
		while(<IN>) {
			$i++;
			my ($no,$y,$m,$d,$time,$sub,$com,$ex1,$w1,$h1,$ex2,$w2,$h2,$ex3,$w3,$h3,$tag,$att,$tube,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$cm) = split(/<>/);
			
			# カレンダ用
			$cal{$d}++;
			
			# 日指定の場合
			next if ($in{d} > 0 && $in{d} != $d);
			
				next if ($i < $pg + 1);
				next if ($i > $pg + $cf{pg_max});
				
				push(@log,$_);
		}
		close(IN);
		
		# ペイジャー作成
		$pager = make_pager($i,$pg);

	}

	# Recent Posts data
	my $topic = read_topic();

	# Recent comments data
	my $comlist = read_com();

	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/mon_calen.html") or error("open err: mon_calen.html");
	my $tmpl = join('',<IN>);
	close(IN);
	
	$tmpl =~ s/!(bbs_title|homepage|cmnurl|[a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/<!-- pager -->/$pager/g;

	if ($cf{rss_make} == 1) {
		$tmpl =~ s|<!-- rss -->|<a href="$cf{rssurl}"><img src="$cf{cmnurl}/rss.png" class="icon"></a>|;
	}

	$tmpl =~ s|<!-- calen -->(.+?)<!-- /calen -->|put_calen($1,$caly,$calm,\%cal,\@archi)|se;
	$tmpl =~ s|<!-- archive -->(.+?)<!-- /archive -->|put_archive($1,\@archi)|se;
	$tmpl =~ s/!topic_count!/$cf{max_topics}/g;
	$tmpl =~ s/<!-- topic -->/$topic/;
	$tmpl =~ s/<!-- pager -->/$pager/g;

	my $prof_img = $set{prof_image}
			? qq|<p><img src="$cf{uplurl}/$set{prof_image}" style="width:$set{prof_img_w}px" alt="プロフ"></p>| : '';

	$set{profile} =~ s/\t/<br>/;
	$tmpl =~ s/!profile!/$set{profile}/g;
	$tmpl =~ s|<!-- prof_image -->|$prof_img|g;

	open(DAT,"$cf{datadir}/resno.dat");
	my $number = join('',<DAT>);
	close(DAT);

	if ($number > 0) {
		$tmpl =~ s/<!-- comlist -->/$comlist/;
		$tmpl =~ s/!comment_count!/$cf{max_com}/g;
	} else {
		$tmpl =~ s/<!-- cmlink -->.+?<!-- cmlink -->//s;
	}

	# プロフィール
	if ($cf{prof} == 0) {
		$tmpl =~ s|<!-- profile -->(.+?)<!-- /profile -->||s;
	}


	# 日付フォーマット
	my $day = "$caly年$calm月";
	$tmpl =~ s/!day!/$day/g;
	
	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ m|(.+)<!-- loop -->(.+?)<!-- /loop -->(.+)|s
			? ($1,$2,$3)
			: error("テンプレート不正");
	
	# ヘッダ表示
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;
	
	# 記事展開
	for (@log) {
		my ($no,$y,$m,$d,$time,$sub,$com,$ex1,$w1,$h1,$ex2,$w2,$h2,$ex3,$w3,$h3,$tag,$att,$tube,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$cm) = split(/<>/);

		if (!$res_cnt{$no}) { $res_cnt{$no} = 0; }

		$com =~ s/\t/<br>/g;
		if ($tag == 1) {
			$com = tag($com);
		} elsif ($cf{autolink}) {
			$com = auto_link($com);
		}
		
		# YouTube
		my $clip;
		if ($att eq 't') {
			$clip = qq|<div class="youtube">| . tag($tube) . qq|</div>| if ($tube);
		
		# UPファイル
		} else {
			$clip = attach($no,$ex1,$w1,$h1,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5);
		}
		$clip &&= "<p>$clip</p>";
		
		# 文字置換
		my $tmp = $loop;
		my $day = "$y/$m/$d $time";
		$tmp =~ s/!date!/$day/g;
		$tmp =~ s/!subject!/$sub/g;
		$tmp =~ s/!comment!/$com/g;
		$tmp =~ s/<!-- clip -->/$clip/g;
		$tmp =~ s/!id!/$no/g;
		$tmp =~ s/!res_com!/$cf{bbs_cgi}?mode=rescom&no=$no/g;
		$tmp =~ s/!res_cnt!/$res_cnt{$no}/g;

		# コメント返信機能ＯＦＦ
		$tmp =~ s/<!-- cmlink -->.+?<!-- cmlink -->//s if (!$cm);
		print $tmp;
	}
	
	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  コメント返信フォーム
#-----------------------------------------------------------
sub rescom_form {

	# コメント数カウント
	my (@log,%res_cnt,@reply,%res,$max_res_flg);
	open(IN,"$cf{rescnt}");
	while(<IN>) {
		my ($no,$y,$m,$d,$res) = split(/<>/);

		$res_cnt{$no} = $res;

	}

	# 表示データ認識
	open(IN,"$cf{datadir}/index.dat") or &error("open err: index.dat");
	while (<IN>) {
		my ($no,$y,$m,$d,$time,$sub,$com,$ex1,$w1,$h1,$ex2,$w2,$h2,$ex3,$w3,$h3,$tag,$att,$tube,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$cm) = split(/<>/);

		if ($in{no} == $no) {

			$res{sub} = $sub;

		push(@log,$_);
		}
	}

	# コメント返信ログ
	open(IN,"$cf{reslog}");
	while(<IN>) {
		my ($no,$ent,$sub,$date,$name,$url,$host,$com,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$chk,$pw,$comsub) = split(/<>/);

		# 該当レス
		if ($in{no} == $ent) {

		push(@reply,$_);
		}
	}
	close(IN);

	# クッキー取得
	my ($c_nam,$c_url,$c_ttl) = get_cookie();
	$c_url ||= 'http://';

	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/com_read.html") or error("open err: com_read.html");
	my $tmpl = join('', <IN>);
	close(IN);

	open(IN,"$cf{tmpldir}/comment.html") or error("open err: comment.html");
	my $resloop = join('', <IN>);
	close(IN);

	# 画像認証作成
	my ($str_plain,$str_crypt);
	if ($cf{use_captcha} > 0) {
		require $cf{captcha_pl};
		($str_plain,$str_crypt) = cap::make($cf{captcha_key},$cf{cap_len});
		$tmpl =~ s/!str_crypt!/$str_crypt/g;
	} else {
		$tmpl =~ s/<!-- captcha_begin -->.+<!-- captcha_end -->//s;
	}
		$tmpl =~ s/!max_res!/$cf{max_res}/g;
		$tmpl =~ s/!c_name!/$c_nam/g;
		$tmpl =~ s/!c_ttl!/$c_ttl/g;
		$tmpl =~ s/!c_url!/$c_url/g;
		$tmpl =~ s/!no!/$in{no}/g;
		$tmpl =~ s/!r-sub!/$res{sub}/g;
		$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;

	# テンプレート分割
	my ($head,$loop,$foot) = split(/<!-- loop -->/,$tmpl,$resloop);

	# 文字置換
	for ($head,$foot) {
		s/!([a-z]+_cgi)!/$cf{$1}/g;
		s|!bbs_css!|$cf{cmnurl}/clip.css|g;
		s|!input_file_btn_css!|$cf{cmnurl}/input_file_btn.css|g;
		s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
		s/!bbs_title!/$cf{bbs_title}/g;

	# 返信フォームのサブミットボタン(ディスエイブル)｛エラーメッセージを表示したい場合は、コメントアウト｝
	#if ($res_cnt{$in{no}} >= $cf{max_res}) {
	#	s|<input class="submit-button" type="submit" value="Submit" />|<input class="submit-button" type="submit" value="Submit" disabled />|g;
	#}

	}

	# 画面展開
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;

	foreach (@log) {
		my ($no,$y,$m,$d,$time,$sub,$com,$ex1,$w1,$h1,$ex2,$w2,$h2,$ex3,$w3,$h3,$tag,$att,$tube,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$cm) = split(/<>/);

		if (!$res_cnt{$no}) { $res_cnt{$no} = 0; }

		$com =~ s/\t/<br>/g;
		if ($tag == 1) {
			$com = tag($com);
		} elsif ($cf{autolink}) {
			$com = auto_link($com);
		}
		
		# YouTube
		my $clip;
		if ($att eq 't') {
			$clip = qq|<div class="youtube">| . tag($tube) . qq|</div>| if ($tube);
		
		# UPファイル
		} else {
			$clip = attach($no,$ex1,$w1,$h1,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5);
		}
		$clip &&= "<p>$clip</p>";

		# 記事書き出し
		my $tmp = $loop;
		$tmp =~ s|!icon:(\w+\.\w+)!|<img src="$cf{cmnurl}/$1" alt="" class="icon" />|g;
		$tmp =~ s/!num!/$no/g;
		$tmp =~ s/!subject!/$sub/g;
		$tmp =~ s|!date!|$y/$m/$d $time|g;
		$tmp =~ s|!comment!|$com|g;
		$tmp =~ s/<!-- clip -->/$clip/g;
		$tmp =~ s|!ico_edit!|<a href="$cf{bbs_cgi}?mode=p_form&no=$no"><img src="$cf{iconurl}/Edit02.png" class="icon" alt="" /></a>|g;
		$tmp =~ s/!res_com!/$cf{bbs_cgi}?mode=rescom&no=$no/g;
		$tmp =~ s/!ico_com!/$cf{ico_com}/g;
		$tmp =~ s|!ico_com02!|<img src="$cf{iconurl}/discuss02.png" class="icon" alt="">|g;
		$tmp =~ s/!res_cnt!/$res_cnt{$no}/g;
		$tmp =~ s/!id!/$no/g;

	# 返信コメント
	if ($res_cnt{$no} == 0) {
		$tmp =~ s|<!-- rescomment -->|<span style="color:#ed6d46">《この記事にはまだコメントがありません》</span>|g;
	} else {
		$tmp =~ s/<!-- rescomment -->//g;
	}

		print $tmp;

	my $i = 0;
	foreach (@reply) {
		my ($no,$ent,$sub,$date,$name,$url,$host,$com,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$chk,$pw,$comsub) = split(/<>/);
		if ($url) { $url = qq |&lt;<a href="$url" target="_blank">URL</a>&gt;|; }

			$com = auto_link($com);

		# UPファイル
		my $resclip;
		$resclip = res_attach($no,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$comsub);
		$resclip &&= "<p>$resclip</p>";

		my $tmp = $resloop;
		$tmp =~ s|!icon:(\w+\.\w+)!|<img src="$cf{cmnurl}/$1" alt="" class="icon" />|g;
		$tmp =~ s|!com_edit!|<a href="$cf{bbs_cgi}?mode=compwd&no=$no"><img src="$cf{iconurl}/edit03.png" alt="" class="icon" /></a>|g;

		$tmp =~ s/!r_name!/$name/g;
		$tmp =~ s/!r_sub!/$comsub/g;
		$tmp =~ s/!r_date!/$date/g;
		$tmp =~ s/!r_url!/$url/g;
		$tmp =~ s/!r_comment!/$com/g;
		$tmp =~ s/<!-- ext -->/$resclip/g;
		print $tmp;
		}
	}
	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  コメントワード検索
#-----------------------------------------------------------
sub res_find_data {
	# 引数取得
	$in{word} =~ s/[\r\n]//g;
	$in{cond} =~ s/\D//g;
	$in{pg} =~ s/\D//g;
	$in{pg} ||= 0;

	# 検索条件プルダウン
	my %op = (1 => 'AND', 0 => 'OR');
	my $op_cond;
	foreach (1,0) {
		if ($in{cond} eq $_) {
			$op_cond .= qq|<option value="$_" selected>$op{$_}\n|;
		} else {
			$op_cond .= qq|<option value="$_">$op{$_}\n|;
		}
	}

	# 検索実行
	my ($i,@reply) = res_search($in{word},$in{cond}) if ($in{word} ne '');
	
	# 検索用
	my $ewd = $in{word};
	$ewd =~ s/(\W)/"%".unpack("H2",$1)/ego;
	$ewd =~ tr/ /+/;
	my $param = "&amp;mode=res_find&amp;word=$ewd&amp;cond=$in{cond}";

	# ページ繰越ボタン作成
	my $pgbtn = make_resbtn($i,$in{pg},$param);

	# 書庫ファイル
	open(IN,"$cf{datadir}/archive.dat") or error("open err: archive.dat");
	my @archi = <IN>;
	close(IN);
	
	chomp @archi;

	# 変数初期化
	my ($caly,$calm,%cal);

	for (@reply) {
		my ($no,$y,$m) = (split(/<>/))[0,1,2];
		$caly = $y;
		$calm = $m;
	}

	# Recent Posts data
	my $topic = read_topic();

	# Recent comments data
	my $comlist = read_com();

	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/com_find.html") or error("open err: com_find.html");
	my $tmpl = join('', <IN>);
	close(IN);

	$tmpl =~ s|<!-- calen -->(.+?)<!-- /calen -->|put_calen($1,$caly,$calm,\%cal,\@archi)|se;
	$tmpl =~ s|<!-- archive -->(.+?)<!-- /archive -->|put_archive($1,\@archi)|se;
	$tmpl =~ s/!topic_count!/$cf{max_topics}/g;
	$tmpl =~ s/<!-- topic -->/$topic/;

	open(DAT,"$cf{datadir}/resno.dat");
	my $number = join('',<DAT>);
	close(DAT);

	if ($number > 0) {
		$tmpl =~ s/<!-- comlist -->/$comlist/;
		$tmpl =~ s/!comment_count!/$cf{max_com}/g;
	} else {
		$tmpl =~ s/<!-- cmlink -->.+?<!-- cmlink -->//s;
	}

	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ m|(.+)<!-- loop -->(.+?)<!-- /loop -->(.+)|s
			? ($1,$2,$3)
			: error("テンプレート不正");

	# 文字置換
	for ($head,$foot) {
		s|!bbs_css!|$cf{cmnurl}/style.css|g;
		s/!bbs_cgi!/$cf{bbs_cgi}/g;
		s/!bbs_title!/$cf{bbs_title}/g;
		s/<!-- op_cond -->/$op_cond/;
		s/!word!/$in{word}/;
		s/!page_btn!/$pgbtn/;

		# ワード検索
		if ($in{word}) {
			s/!result!/$i/g;
		} else {
			s|<!-- find -->.+?<!-- /find -->||sg;
		}
	}

	# 画面展開
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;

	foreach (@reply) {
		my ($no,$ent,$sub,$date,$name,$url,$host,$com,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$chk,$pw,$comsub) = split(/<>/);
		if ($url) { $url = qq |&lt;<a href="$url" target="_blank">URL</a>&gt;|; }

		if ($url) { $url = qq |&lt;<a href="$url" target="_blank">URL</a>&gt;|; }
			$com = auto_link($com);

		# UPファイル
		my $resclip;
			$resclip = res_attach($no,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$comsub);
			$resclip &&= "<p>$resclip</p>";

		# 記事書き出し
		my $tmp = $loop;
		$tmp =~ s|!icon:(\w+\.\w+)!|<img src="$cf{cmnurl}/$1" alt="" class="icon" />|g;
		$tmp =~ s|!com_edit!|<a href="$cf{bbs_cgi}?mode=compwd&no=$no"><img src="$cf{iconurl}/edit03.png" alt="" class="icon" /></a>|g;
		$tmp =~ s/!r_num!/$no/g;
		$tmp =~ s/!r_name!/$name/g;
		$tmp =~ s/!r_sub!/$comsub/g;
		$tmp =~ s/!r_date!/$date/g;
		$tmp =~ s/!r_url!/$url/g;
		$tmp =~ s/!r_comment!/$com/g;
		$tmp =~ s/<!-- ext -->/$resclip/g;
		print $tmp;
	}

	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  コメント検索実行
#-----------------------------------------------------------
sub res_search {
	my ($word,$cond) = @_;

	# キーワードを配列化
	$word =~ s/\x81\x40/ /g;
	my @wd = split(/\s+/,$word);

	# UTF-8定義
	my $byte1 = '[\x00-\x7f]';
	my $byte2 = '[\xC0-\xDF][\x80-\xBF]';
	my $byte3 = '[\xE0-\xEF][\x80-\xBF]{2}';
	my $byte4 = '[\xF0-\xF7][\x80-\xBF]{3}';

	# 検索処理
	my ($i,@reply);
	open(IN,"$cf{reslog}") or error("open err: $cf{logfile}");
	while (<IN>) {
		my ($no,$ent,$sub,$date,$name,$url,$host,$com,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$chk,$pw,$comsub) = split(/<>/);

		my $flg;
		foreach my $wd (@wd) {
			if ("$date $name $url $com" =~ /^(?:$byte1|$byte2|$byte3|$byte4)*?\Q$wd\E/i) {
				$flg++;
				if ($cond == 0) { last; }
			} else {
				if ($cond == 1) { $flg = 0; last; }
			}
		}
		next if (!$flg);

		$i++;
		next if ($i < $in{pg} + 1);
		next if ($i > $in{pg} + $cf{max_resfind});
		
		push(@reply,$_);
	}
	close(IN);

	# 検索結果
	return ($i,@reply);
}

#-----------------------------------------------------------
#  コメント返信パスワードフォーム
#-----------------------------------------------------------
sub com_pwdform {

	$in{no} =~ s/\D//g;

	# クッキー取得
	my @cook = get_cookie();
	$cook[2] ||= 'http://';

	# 記事
	my ($name,$pwd);
	open(DAT,"+< $cf{reslog}") || &error("Open Error: $cf{reslog}");
	eval "flock(DAT, 2);";
	while (<DAT>) {
		my ($no,$ent,$sub,$date,$name,$url,$host,$com,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$chk,$pw,$comsub) = split(/<>/);

	if ($in{no} == $no) {
		$name = $name;
		$sub = $sub;
		$pwd = $pw;
		last;
		}
	}
	close(DAT);

	# パスワード未設定
	&error("このコメントはパスワード未設定のため編集できません") if ($pwd eq '');

	my ($no,$ent,$sub,$date,$name,$url,$host,$com,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$chk,$pw,$comsub) = split(/<>/);
		$url &&= qq|<a href="$url" target="_blank"><img src="$cf{ico_home}" class="icon"></a>|;
		$com = auto_link($com) if ($cf{autolink});
		if (length($com) > 110) { $com = substr($com,0,120); $com .= '.....(略)'; }

		# UPファイル
		my $resclip;
			$resclip = res_attach($no,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$comsub);
			$resclip &&= "<p>$resclip</p>";

	my @col = split(/\s+/, $cf{colors});

	open(IN,"$cf{tmpldir}/com_pwdform.html") or &error("open err: com_pwdform.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# 文字置換
	$tmpl =~ s/!sub!/$sub/g;
	$tmpl =~ s/!name!/$name/g;
	$tmpl =~ s/!comsub!/$comsub/g;
	$tmpl =~ s/!no!/$in{no}/g;
	$tmpl =~ s|!comment!|$com|g;
	$tmpl =~ s/!date!/$date/g;
	$tmpl =~ s/!url!/$url/g;
	$tmpl =~ s/<!-- ext -->/$resclip/g;
	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/clip.css|g;
	$tmpl =~ s|!mob_css!|$cf{cmnurl}/clip-mobile.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
	$tmpl =~ s/!num!/$no/g;
	$tmpl =~ s/!pwd!/$pwd/g;

	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;

	# フッタ
	footer();

	exit;
}

#-----------------------------------------------------------
#  ワード検索
#-----------------------------------------------------------
sub find_log {
	# 条件
	$in{cond} =~ s/\D//g;
	$in{word} =~ s/\t//g;
	
	# 検索条件プルダウン
	my %op = (1 => 'AND', 0 => 'OR');
	my $op_cond;
	for (1,0) {
		if ($in{cond} eq $_) {
			$op_cond .= qq|<option value="$_" selected>$op{$_}</option>\n|;
		} else {
			$op_cond .= qq|<option value="$_">$op{$_}</option>\n|;
		}
	}
	
	# 検索実行
	my ($hit,@log) = search($in{word},$in{cond}) if ($in{word} ne '');

	# プロフィール認識
	my %set = read_profile();

	# コメント数カウント
	my %res_cnt;
	open(IN,"$cf{rescnt}");
	while(<IN>) {
		my ($no,$y,$m,$d,$res) = split(/<>/);

		$res_cnt{$no} = $res;

	}

	# 書庫ファイル
	open(IN,"$cf{datadir}/archive.dat") or error("open err: archive.dat");
	my @archi = <IN>;
	close(IN);
	
	chomp @archi;

	# 変数初期化
	my ($caly,$calm,%cal);

	for (@log) {
		my ($no,$y,$m) = (split(/<>/))[0,1,2];
		$caly = $y;
		$calm = $m;
	}

	# Recent Posts data
	my $topic = read_topic();

	# Recent comments data
	my $comlist = read_com();

	# テンプレート
	open(IN,"$cf{tmpldir}/find.html") or error("open err: find.html");
	my $tmpl = join('',<IN>);
	close(IN);
	
	$tmpl =~ s/!(bbs_title|homepage|cmnurl|[a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/<!-- op_cond -->/$op_cond/;
	$tmpl =~ s/!word!/$in{word}/;
	$tmpl =~ s|<!-- calen -->(.+?)<!-- /calen -->|put_calen($1,$caly,$calm,\%cal,\@archi)|se;
	$tmpl =~ s|<!-- archive -->(.+?)<!-- /archive -->|put_archive($1,\@archi)|se;
	$tmpl =~ s/!topic_count!/$cf{max_topics}/g;
	$tmpl =~ s/<!-- topic -->/$topic/;

	my $prof_img = $set{prof_image}
			? qq|<p><img src="$cf{uplurl}/$set{prof_image}" style="width:$set{prof_img_w}px" alt="プロフ"></p>| : '';

	$set{profile} =~ s/\t/<br>/;
	$tmpl =~ s/!profile!/$set{profile}/g;
	$tmpl =~ s|<!-- prof_image -->|$prof_img|g;

	# プロフィール
	if ($cf{prof} == 0) {
		$tmpl =~ s|<!-- profile -->(.+?)<!-- /profile -->||s;
	}

	open(DAT,"$cf{datadir}/resno.dat");
	my $number = join('',<DAT>);
	close(DAT);

	if ($number > 0) {
		$tmpl =~ s/<!-- comlist -->/$comlist/;
		$tmpl =~ s/!comment_count!/$cf{max_com}/g;
	} else {
		$tmpl =~ s/<!-- cmlink -->.+?<!-- cmlink -->//s;
	}

	# ワード検索
	if ($in{word}) {
		$tmpl =~ s/!result!/$hit/g;
	} else {
		$tmpl =~ s|<!-- find -->.+?<!-- /find -->||sg;
	}
	
	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ m|(.+)<!-- loop -->(.+?)<!-- /loop -->(.+)|s
			? ($1,$2,$3)
			: error("テンプレート不正");
	
	# ヘッダ部
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;
	
	# ループ部
	for (@log) {
		my ($no,$y,$m,$d,$time,$sub,$com,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$tag,$att,$tube,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$cm) = split(/<>/);
		$com =~ s/\t/<br>/g;
		if ($tag == 1) {
			$com = tag($com);
		} elsif ($cf{autolink}) {
			$com = auto_link($com);
		}
		
		# YouTube
		my $clip;
		if ($att eq 't') {
			$clip = tag($tube) if ($tube);
		
		# UPファイル
		} else {
			$clip = attach($no,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5);
		}
		$clip &&= "<p>$clip</p>";
		
		my $tmp = $loop;
		$tmp =~ s/!subject!/$sub/g;

		my $day = "$y/$m/$d $time";
		$tmp =~ s/!date!/$day/g;

		$tmp =~ s/!comment!/$com/g;
		$tmp =~ s/<!-- clip -->/$clip/g;
		$tmp =~ s/!res_cnt!/$res_cnt{$no}/g;

		# コメント返信機能ＯＦＦ
		$tmp =~ s/<!-- cmlink -->.+?<!-- cmlink -->//s if (!$cm);
		print $tmp;
	}
	
	# フッタ部
	footer($foot);
}

#-----------------------------------------------------------
#  検索実行
#-----------------------------------------------------------
sub search {
	my ($word,$cond) = @_;
	
	# キーワードを配列化
	$word =~ s/　/ /g;
	my @wd = split(/\s+/,$word);
	
	# UTF-8定義
	my $byte1 = '[\x00-\x7f]';
	my $byte2 = '[\xC0-\xDF][\x80-\xBF]';
	my $byte3 = '[\xE0-\xEF][\x80-\xBF]{2}';
	my $byte4 = '[\xF0-\xF7][\x80-\xBF]{3}';
	
	# 検索処理
	my ($i,@log);
	open(IN,"$cf{datadir}/index.dat");
	while (<IN>) {
		my ($no,$y,$m,$d,$time,$sub,$com,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$tag,$att,$tube,$ex4,$w4,$h4,$ex5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$cm) = split(/<>/);
		
		my $flg;
		for my $wd (@wd) {
			if ("$sub $com" =~ /^(?:$byte1|$byte2|$byte3|$byte4)*?\Q$wd\E/i) {
				$flg++;
				if ($cond == 0) { last; }
			} else {
				if ($cond == 1) { $flg = 0; last; }
			}
		}
		next if (!$flg);

		$i++;
		next if ($i < $in{pg} + 1);
		next if ($i > $in{pg} + $cf{max_find});
		
		push(@log,$_);
	}
	close(IN);
	
	# 検索結果
	return ($i,@log);
}

#-----------------------------------------------------------
#  ペイジャー作成
#-----------------------------------------------------------
sub make_pager {
	my ($i,$pg) = @_;
	
	# ページ繰越定義
	my $next = $pg + $cf{pg_max};
	my $back = $pg - $cf{pg_max};
	
	# ページ繰越ボタン作成
	my @pg;
	if ($back >= 0 or $next < $i) {
		my $flg;
		my ($w,$x,$y,$z) = (0,1,0,$i);
		while ($z > 0) {
			if ($pg == $y) {
				$flg++;
				push(@pg,qq!<span class="page active">$x</span>\n!);
			} else {
				push(@pg,qq!<a href="$cf{bbs_cgi}?pg=$y" class="page gradient">$x</a>\n!);
			}
			$x++;
			$y += $cf{pg_max};
			$z -= $cf{pg_max};
			
			if ($flg) { $w++; }
			last if ($w >= 5 && @pg >= 10);
		}
	}
	while( @pg >= 11 ) { shift(@pg); }
	my $ret = join('',@pg);
	if ($back >= 0) {
		$ret = qq!<a href="$cf{bbs_cgi}?pg=$back" class="page gradient">&laquo;</a>\n! . $ret;
	}
	if ($next < $i) {
		$ret .= qq!<a href="$cf{bbs_cgi}?pg=$next" class="page gradient">&raquo;</a>\n!;
	}
	# 結果を返す
	return $ret ? qq|<div class="pagination">\n$ret</div>| : '';
}

#-----------------------------------------------------------
#  コメント繰越ボタン作成
#-----------------------------------------------------------
sub make_resbtn {
	my ($i,$pg,$param) = @_;

	# ページ繰越数定義
	$cf{max_resfind} ||= 10;
	my $next = $pg + $cf{max_resfind};
	my $back = $pg - $cf{max_resfind};

	# ページ繰越ボタン作成
	my @pg;
	if ($back >= 0 || $next < $i) {
		my $flg;
		my ($w,$x,$y,$z) = (0,1,0,$i);
		while ($z > 0) {
			if ($pg == $y) {
				$flg++;
				push(@pg,qq!<li><span>$x</span></li>!);
			} else {
				push(@pg,qq!<li><a href="$cf{bbs_cgi}?pg=$y$param">$x</a></li>!);
			}
			$x++;
			$y += $cf{max_resfind};
			$z -= $cf{max_resfind};

			if ($flg) { $w++; }
			last if ($w >= 5 && @pg >= 10);
		}
	}
	while( @pg >= 11 ) { shift(@pg); }
	my $ret = join('', @pg);
	if ($back >= 0) {
		$ret = qq!<li><a href="$cf{bbs_cgi}?pg=$back$param">&laquo;</a></li>\n! . $ret;
	}
	if ($next < $i) {
		$ret .= qq!<li><a href="$cf{bbs_cgi}?pg=$next$param">&raquo;</a></li>\n!;
	}
	
	# 結果を返す
	return $ret ? qq|<ul class="pager">\n$ret</ul>| : '';
}

#-----------------------------------------------------------
#  クッキー取得
#-----------------------------------------------------------
sub get_cookie {
	# クッキー取得
	$ENV{HTTP_COOKIE} =~ /$cf{cookie_id}=(.+);?/;
	my $cookie = $1;

	# URLデコード
	my @cook;
	foreach ( split(/<>/,$cookie) ) {
		s/%([0-9A-Fa-f][0-9A-Fa-f])/pack("H2", $1)/eg;
		s/[&"'<>]//g;

		push(@cook,$_);
	}
	return @cook;
}

#-----------------------------------------------------------
#  フッター
#-----------------------------------------------------------
sub footer {
	my $foot = shift;

	# 著作権表記（削除・改変禁止）
	my $copy = <<EOM;
<div id="footer">
	- <a href="https://www.kent-web.com/bbs/topics/topics.cgi" target="_top">TopicsBoard</a> -<br>
	- <a href="https://www.transformsite.com/" target="_top">$cf{version}</a> -<br />
	++ <a href="https://www.transformsite.com/" target="_top">Edited by TRANSFORM</a> ++
</div>
EOM

	if ($foot =~ /(.+)(<\/body[^>]*>.*)/si) {
		print "$1$copy$2\n";
	} else {
		print "$foot$copy\n";
		print "</body></html>\n";
	}
	exit;
}

#-----------------------------------------------------------
#  自動リンク
#-----------------------------------------------------------
sub auto_link {
	my $text = shift;
	
	$text =~ s/(s?https?:\/\/([\w-.!~*'();\/?:\@=+\$,%#]|&amp;)+)/<a href="$1" target="_blank">$1<\/a>/g;
	return $text;
}

#-----------------------------------------------------------
#  タグ復元
#-----------------------------------------------------------
sub tag {
	local($_) = @_;
	
	# 変換
	s/&lt;/</g;
	s/&gt;/>/g;
	s/&amp;/&/g;
	s/&quot;/"/g;
	$_;
}

#-----------------------------------------------------------
#  添付表示
#-----------------------------------------------------------
sub attach {
	my $no = shift;
	my (%e,%w,%h,%imgtxt);
	($e{1},$w{1},$h{1},$e{2},$w{2},$h{2},$e{3},$w{3},$h{3},$e{4},$w{4},$h{4},$e{5},$w{5},$h{5},$imgtxt{1},$imgtxt{2},$imgtxt{3},$imgtxt{4},$imgtxt{5}) = @_;

	my $ret;
	for my $i (1 .. 5) {
		next if (!$e{$i});
		
		# 指定アイコン
		if (defined($cf{icons}{$e{$i}})) {
			my $size = -s "$cf{upldir}/$no-$i$e{$i}";
			my $size = get_size("$cf{upldir}/$no-$i$e{$i}");
			$ret .= qq|<div class="addclip"><a href="$cf{uplurl}/$no-$i$e{$i}" target="_blank"><div class="attachment"><img src="$cf{icourl}/$cf{icons}{$e{$i}}" alt="" /></a> (<b style="color:#00a1e9">$size</b>)<br />$imgtxt{$i}</div></div>|;

		# 画像の場合
		} elsif ($e{$i} =~ /^\.(jpg|gif|png)$/) {
			if (-f "$cf{upldir}/$no-s-$i$e{$i}") {
				$ret .= qq|<a href="$cf{uplurl}/$no-$i$e{$i}" class="fancybox" data-fancybox-group="gallery" title="No.$no, ファイル名：$imgtxt{$i}"><div class="snap"><img src="$cf{uplurl}/$no-s-$i$e{$i}" class="img" alt="$no-s-$i$e{$i}" class="snapshot" /></a><br />$imgtxt{$i}</div>|;
			} else {
				my ($w,$h) = resize($w{$i},$h{$i});
				$ret .= qq|<a href="$cf{uplurl}/$no-$i$e{$i}" class="fancybox" data-fancybox-group="gallery" title="No.$no, ファイル名：$imgtxt{$i}"><div class="snap"><img src="$cf{uplurl}/$no-$i$e{$i}" width="$w" height="$h" class="img" alt="$no-$i$e{$i}" class="snapshot" /></a><br />$imgtxt{$i}</div>|;
			}
		}
	}
	return $ret;
}

#-----------------------------------------------------------
#  コメント記事添付表示
#-----------------------------------------------------------
sub res_attach {
	my $no = shift;
	my (%e,%w,%h,$sub);
	($e{1},$w{1},$h{1},$e{2},$w{2},$h{2},$e{3},$w{3},$h{3},$sub) = @_;

	my $ret;
	for my $i (1 .. 3) {
		next if (!$e{$i});
		
		# 指定アイコン
		if (defined($cf{icons}{$e{$i}})) {
			my $size = -s "$cf{r_upldir}/$no-$i$e{$i}";
			$size = int(($size / 1024) + 0.5) . 'KB';
			$ret .= qq|<a href="$cf{r_uplurl}/$no-$i$e{$i}" target="_blank"><img src="$cf{iconurl}/$cf{icons}{$e{$i}}" alt="" class="icon" /></a> ($size)\n|;

		# 画像の場合
		} elsif ($e{$i} =~ /^\.(jpg|gif|png)$/) {
			if (-f "$cf{r_upldir}/$no-s-$i$e{$i}") {
				$ret .= qq|<a href="$cf{r_uplurl}/$no-$i$e{$i}" class="fancybox" data-fancybox-group="gallery" title="No.$no"><img src="$cf{r_uplurl}/$no-s-$i$e{$i}" class="img" alt="$no-s-$i$e{$i}" /></a>|;
			} else {
				my ($w,$h) = resize_res($w{$i},$h{$i});
				$ret .= qq|<a href="$cf{r_uplurl}/$no-$i$e{$i}" class="fancybox" data-fancybox-group="gallery" title="No.$no"><img src="$cf{r_uplurl}/$no-$i$e{$i}" width="$w" height="$h" class="img" alt="$no-$i$e{$i}" /></a>|;
			}
		# 画像以外
		} elsif ($e{$i} =~ /^\.(jpg|gif|png)$/) {
			my $size = -s "$cf{r_upldir}/$no-$i$e{$i}";
			$size = int(($size / 1024) + 0.5) . 'KB';
			$ret .= qq|[添付]: <a href="$cf{r_uplurl}/$no-$i$e{$i}" target="_blank">$no-$i$e{$i}</a> ($size)\n|;
		}
	}
	return $ret;
}

#-----------------------------------------------------------
#  ファイル容量
#-----------------------------------------------------------
sub get_size {
	my $file = shift;
	
	my $size = -s $file || 0;
	return $size > 1024000 ? comma(int($size/1024000)) . 'MB' : $size > 1024 ? comma(int($size/1024)) . 'KB' : $size . 'Bytes';
}

#-----------------------------------------------------------
#  桁区切り
#-----------------------------------------------------------
sub comma {
	local($_) = @_;

	1 while s/(.*\d)(\d\d\d)/$1,$2/;
	$_;
}

#-----------------------------------------------------------
#  エラー画面
#-----------------------------------------------------------
sub error {
	my $err = shift;
	
	open(IN,"$cf{tmpldir}/error.html") or die;
	my $tmpl = join('',<IN>);
	close(IN);
	
	$tmpl =~ s/!(bbs_title|homepage|cmnurl|[a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/!error!/$err/g;
	$tmpl =~ s/!cmnurl!/$cf{cmnurl}/;
	
	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;

	# フッタ
	footer();

	exit;
}

