#┌─────────────────────────────────
#│ NEWS BOOK v1.0 : make_rss.pl - 2023/07/17
#│ copyright (c) TRANSFORM, Since 2012
#│ https://www.transformsite.com/
#└─────────────────────────────────

# モジュール宣言
use strict;
use CGI::Carp qw(fatalsToBrowser);
use lib "./lib";
use CGI::Minimal;

# データ受理
CGI::Minimal::max_read_size($cf{maxdata});
my $cgi = CGI::Minimal->new;
error('容量オーバー') if ($cgi->truncated);
my %in = parse_rssform($cgi);

# 設定ファイル認識
require "./init.cgi";
my %cf = set_init();

#-----------------------------------------------------------
#  RSS 2.0 作成
#-----------------------------------------------------------
sub make_rss {
	my @log = @_;
	
	# RSS
	my $rss = <<EOM;
<?xml version="1.0" encoding="UTF-8" ?>
<?xml-stylesheet href="styles-rss2.css" type="text/css"?>
<rss version="2.0">
	<channel>
		<title>$cf{bbs_rssttl}</title>
		<link>$cf{indexurl}</link>
		<description>$cf{description}</description>
		<language>ja</language>
		<generator>$cf{version}</generator>
EOM

	# 月/週
	#my @mon  = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
	my @mon  = qw(1 2 3 4 5 6 7 8 9 10 11 12);
	my @week = qw(Sun Mon Tue Wed Thu Fri Sat);
	
	my $i;
	for (@log) {
		my ($no,$y,$m,$d,$time,$sub,$com,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$tag,$att,$tube,$e4,$w4,$h4,$e5,$w5,$h5,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$cm) = split(/<>/);
		
		# 曜日
		my $w = get_week($d,$m,$y);
		
		# 文字カット: description
		my $char = strcutbytes_utf8($com,150);

		#$com = chg_rsstag($com);
		#$com =~ s/&lt;.*?&gt;//g;
		$com =~ s/\t/\n/g;

		my $clip = rss_att($no,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3,$e4,$w4,$h4,$e5,$w5,$h5);
		$clip &&= "<p>$clip</p>";
		
		$rss .= qq|\t\t<item>\n|;
		$rss .= qq|\t\t\t<title>$sub</title>\n|;
		$rss .= qq|\t\t\t<link>$cf{rsslink}?mode=topic&amp;no=$no&amp;ym=$y$m</link>\n|;
		$rss .= qq|\t\t\t<description>$com \n\n$clip</description>\n|;
		$rss .= qq|\t\t\t<pubDate>$y/$mon[$m-1]/$d ($week[$w]) $time</pubDate>\n|;
		$rss .= qq|\t\t</item>\n|;
		
		$i++;
		last if ($i >= $cf{pg_max});
	}
	$rss .= "\t</channel>\n</rss>\n";
	
	# RSS書込
	open(RSS,"> $cf{rssfile}") or error("write err: $cf{rssfile}");
	print RSS $rss;
	close(RSS);
}

#-----------------------------------------------------------
#  添付表示
#-----------------------------------------------------------
sub rss_att {
	my $no = shift;
	my (%e,%w,%h);
	($e{1},$w{1},$h{1},$e{2},$w{2},$h{2},$e{3},$w{3},$h{3},$e{4},$w{4},$h{4},$e{5},$w{5},$h{5}) = @_;

	my $ret;
	for my $i (1 .. 5) {
		next if (!$e{$i});
		
		# 指定アイコン
		if (defined($cf{icons}{$e{$i}})) {
			my $size = -s "$cf{rss_imgdir}/$no-$i$e{$i}";
			$size = int(($size / 1024) + 0.5) . 'KB';
			$ret .= qq|&lt;a href="$cf{rss_imgurl}/$no-$i$e{$i}" target="_blank"&gt;&lt;img src="$cf{rss_iconurl}/$cf{icons}{$e{$i}}" alt="" style="vertical-align: middle; border: none;" /&gt;&lt;/a&gt; ($size)\n|;

		# 画像の場合
		} elsif ($e{$i} =~ /^\.(jpg|gif|png)$/) {
			if (-f "$cf{rss_imgdir}/$no-s-$i$e{$i}") {
				$ret .= qq|&lt;a href="$cf{rss_imgurl}/$no-$i$e{$i}" target="_blank"&gt;&lt;img src="$cf{rss_imgurl}/$no-s-$i$e{$i}" style="vertical-align: middle; border: none;" alt="$no-s-$i$e{$i}" /&gt;&lt;/a&gt;|;
			} else {
				my ($w,$h) = resize($w{$i},$h{$i});
				$ret .= qq|&lt;a href="$cf{rss_imgurl}/$no-$i$e{$i}" target="_blank"&gt;&lt;img src="$cf{rss_imgurl}/$no-$i$e{$i}" width="$w" height="$h" style="vertical-align: middle; border: none;" alt="$no-$i$e{$i}" /&gt;&lt;/a&gt;|;
			}
		# 画像以外
		} else {
			my $size = -s "$cf{rss_imgdir}/$no-$i$e{$i}";
			$size = int(($size / 1024) + 0.5) . 'KB';
			$ret .= qq|[添付]: &lt;a href="$cf{rss_imgurl}/$no-$i$e{$i}" target="_blank"&gt;$no-$i$e{$i}&lt;/a&gt; ($size)\n|;
		}
	}
	return $ret;
}

#-----------------------------------------------------------
#  画像リサイズ
#-----------------------------------------------------------
sub resize {
	my ($w,$h) = @_;

	# 画像表示縮小
	if ($w > $cf{max_img_w} || $h > $cf{max_img_h}) {
		my $w2 = $cf{max_img_w} / $w;
		my $h2 = $cf{max_img_h} / $h;
		my $key;
		if ($w2 < $h2) { $key = $w2; } else { $key = $h2; }
		$w = int ($w * $key) || 1;
		$h = int ($h * $key) || 1;
	}
	return ($w,$h);
}

#-----------------------------------------------------------
#  文字カット for UTF-8
#  [quote] http://linkage.white-void.net/development/server/perl-utf8-strcut.html
#-----------------------------------------------------------
sub strcutbytes_utf8 {
	my ($src,$maxlen) = @_;
	$src =~ s/[\t\n]//g;
	$src =~ s/&lt;.*?&gt;//g;
	
	my $srclen = length($src);
	my $srcpos = 0;
	while($srcpos < $srclen) {
		my $character = substr($src, $srcpos, 1);
		my $value = ord($character);
		if($value < 0x80) { # ASCII characters
			$srcpos++;
			next;
		}
		my $width = 6;
		$width = 5 if ($value < 0xFC);
		$width = 4 if ($value < 0xF8);
		$width = 3 if ($value < 0xF0);
		$width = 2 if ($value < 0xE0);
		my $nextpos = $srcpos + $width;
		last if($nextpos > $maxlen);
		last if($nextpos > $srclen); # sequence is incomplete
		$srcpos = $nextpos;
	}
	return substr($src, 0, $srcpos);
}

#-----------------------------------------------------------
#  フォームデコード
#-----------------------------------------------------------
sub parse_rssform {
	my $cgi = shift;
	
	my %in;
	for ( $cgi->param() ) {
		my $val = $cgi->param($_);
		
		if (!/^upfile\d$/) {
			# 無効化
			$val =~ s/&amp;/&/g;
			$val =~ s/&lt;/</g;
			$val =~ s/&gt;/>/g;
			$val =~ s/&quot;/"/g;
			$val =~ s/&#39;/'/g;
			
			# 改行変換
			$val =~ s/\t/\r\n/g;
			$val =~ s/\t/\n/g;
			$val =~ s/\t/\r/g;
		}
		$in{$_} = $val;
	}
	return %in;
}

#-----------------------------------------------------------
#  ツェラー公式
#-----------------------------------------------------------
sub get_week {
	my ($day,$month,$year) = @_;
	if ($month == 1 || $month == 2) {
		$year--;
		$month += 12;
	}
	return int($year + int($year/4) - int($year/100) + int($year/400) + int((13*$month+8)/5) + $day) % 7;
}

#-----------------------------------------------------------
#  タグ復元
#-----------------------------------------------------------
sub chg_rsstag {
	local($_) = @_;
	
	s/&amp;/&/g;
	s/&quot;/"/g;
	s/&lt;/</g;
	s/&gt;/>/g;
	s/&#39;/'/g;
	$_;
}


1;

